@@ -9,7 +9,7 @@
  #  Note        : Include this file in your C++ source code, if you
  #                want to use the G'MIC interpreter in your own program.
  #
- #  Copyright   : David Tschumperle
+ #  Copyright   : David Tschumperlé
  #                ( https://tschumperle.users.greyc.fr/ )
  #
  #  Licenses    : This file is 'dual-licensed', you have to choose one
@@ -52,7 +52,7 @@
 */
 
 #ifndef gmic_version
-#define gmic_version 281
+#define gmic_version 291
 
 #ifndef gmic_pixel_type
 #define gmic_pixel_type float
@@ -78,19 +78,29 @@ namespace cimg_library {
     unsigned int _height;      // Number of image lines (dimension along the Y-axis)
     unsigned int _depth;       // Number of image slices (dimension along the Z-axis)
     unsigned int _spectrum;    // Number of image channels (dimension along the C-axis)
-    bool _is_shared;           // Tells if the data buffer is shared by another structure
+    bool _is_shared;           // Tells if the data buffer has been allocated by another object
     T *_data;                  // Pointer to the first pixel value
 
     // Destructor.
-    ~gmic_image();
+    ~gmic_image() {
+      if (!_is_shared) delete[] _data;
+    }
 
     // Constructor.
-    gmic_image():_width(0),_height(0),_depth(0),_spectrum(0),_is_shared(false),_data(0) {}
+    gmic_image():_width(0),_height(0),_depth(0),_spectrum(0),_is_shared(false),_data(0) { }
 
     // Allocate memory for specified image dimensions.
     gmic_image<T>& assign(const unsigned int size_x, const unsigned int size_y=1,
                           const unsigned int size_z=1, const unsigned int size_c=1);
 
+    // Create image by copying existing buffer of t values.
+    template<typename t>
+    gmic_image<T>& assign(const t *const values, const unsigned int size_x, const unsigned int size_y=1,
+                          const unsigned int size_z=1, const unsigned int size_c=1);
+
+    gmic_image<T>& assign(const T *const values, const unsigned int size_x, const unsigned int size_y,
+                          const unsigned int size_z, const unsigned int size_c, const bool is_shared);
+
     // Pixel access.
     operator T*() {
       return _data;
@@ -117,7 +127,9 @@ namespace cimg_library {
     gmic_image<T> *_data;          // Pointer to the first image of the list
 
     // Destructor.
-    ~gmic_list();
+    ~gmic_list() {
+      delete[] _data;
+    }
 
     // Constructor.
     gmic_list():_width(0),_allocated_width(0),_data(0) {}
@@ -155,12 +167,12 @@ namespace cimg_library {
 
 #ifndef cimg_verbosity
 #define cimg_verbosity 1
-#endif // #ifndef cimg_verbosity
+#endif
 
 #ifdef _MSC_VER
 #pragma comment(linker,"/STACK:6291456")
 #pragma inline_depth(2)
-#endif // #ifdef _MSC_VER
+#endif
 
 #include <locale>
 #ifdef cimg_version
@@ -173,24 +185,41 @@ namespace cimg_library {
 inline bool *gmic_abort_ptr(bool *const p_is_abort);
 #define cimg_abort_init bool *const gmic_is_abort = ::gmic_abort_ptr(0)
 #define cimg_abort_test if (*gmic_is_abort) throw CImgAbortException()
-#endif // #ifdef cimg_use_abort
+#endif
 
 template<typename T>
-inline double gmic_mp_call(char *const str, void *const p_list, const T& pixel_type);
-#define cimg_mp_call_function(str) return ::gmic_mp_call(str,&mp.listout,(T)0)
+inline double gmic_mp_run(char *const str,
+                          void *const p_list, const T& pixel_type);
+#define cimg_mp_func_run(str) \
+  return ::gmic_mp_run(str,&mp.listout,(T)0)
 
 template<typename Ts, typename T>
 inline double gmic_mp_store(const Ts *const ptr,
                             const unsigned int w, const unsigned int h, const unsigned int d, const unsigned int s,
-                            const char *const str, void *const p_list, const T& pixel_type);
-#define cimg_mp_store_function(ptr,w,h,d,s,str) return ::gmic_mp_store(ptr,w,h,d,s,str,&mp.listout,(T)0)
+                            const bool is_compressed, const char *const str,
+                            void *const p_list, const T& pixel_type);
+#define cimg_mp_func_store(ptr,w,h,d,s,is_compressed,str) \
+  return ::gmic_mp_store(ptr,w,h,d,s,is_compressed,str,&mp.listout,(T)0)
+
+template<typename T, typename Ts>
+inline double gmic_mp_name(const unsigned int ind, Ts *const out_str, const unsigned int siz,
+                           void *const p_list, const T& pixel_type);
+#define cimg_mp_func_name(ind,out_str,siz) \
+  return ::gmic_mp_name(ind,out_str,siz,&mp.listout,(T)0)
+
+template<typename T>
+inline double gmic_mp_setname(const unsigned int ind, const char *const str,
+                              void *const p_list, const T& pixel_type);
+#define cimg_mp_func_setname(ind,str) \
+  return ::gmic_mp_setname(ind,str,&mp.listout,(T)0)
+
 #ifndef cimg_display
 #define cimg_display 0
-#endif // #ifndef cimg_display
+#endif
 #ifndef cimg_appname
 #define cimg_appname "gmic"
-#endif // #ifndef cimg_appname
-#include "./CImg.h"
+#endif
+#include "CImg.h"
 
 #if cimg_OS==2
 #include <process.h>
@@ -201,6 +230,7 @@ inline double gmic_mp_store(const Ts *const ptr,
 #include <sys/resource.h>
 #include <sys/syscall.h>
 #include <signal.h>
+
 #endif // #if cimg_OS==2
 
 // Define some special character codes used for replacement in double quoted strings.
@@ -209,6 +239,19 @@ const char gmic_dollar = 23, gmic_lbrace = 24, gmic_rbrace = 25, gmic_comma = 26
 
 #endif // #ifndef gmic_build
 
+// Define gmic_uint64 type.
+#ifndef gmic_uint64
+#if cimg_OS==2
+#define gmic_uint64 __int64
+#else // #if cimg_OS==2
+#if UINTPTR_MAX==0xffffffff || defined(__arm__) || defined(_M_ARM) || ((ULONG_MAX)==(UINT_MAX))
+#define gmic_uint64 unsigned long long
+#else
+#define gmic_uint64 unsigned long
+#endif // #if UINTPTR_MAX==0xffffffff || defined(__arm__) || defined(_M_ARM) || ((ULONG_MAX)==(UINT_MAX))
+#endif // #if cimg_OS==2
+#endif // #ifndef gmic_uint64
+
 // Define main libgmic class 'gmic'.
 //----------------------------------
 #define gmic_image cimg_library::CImg
@@ -255,15 +298,23 @@ struct gmic {
   template<typename T>
   static bool search_sorted(const char *const str, const T& list, const unsigned int length, unsigned int &out_ind);
   template<typename T>
-  static double mp_call(char *const str, void *const p_list, const T& pixel_type);
+  static double mp_run(char *const str,
+                       void *const p_list, const T& pixel_type);
   template<typename Ts, typename T>
   static double mp_store(const Ts *const ptr,
                          const unsigned int w, const unsigned int h, const unsigned int d, const unsigned int s,
-                         const char *const str, void *const p_list, const T& pixel_type);
+                         const bool is_compressed, const char *const str,
+                         void *const p_list, const T& pixel_type);
+  template<typename T, typename Ts>
+  static double mp_name(const unsigned int ind, Ts *const out_str, const unsigned int siz,
+                        void *const p_list, const T& pixel_type);
+  template<typename T>
+  static double mp_setname(const unsigned int ind, const char *const str,
+                           void *const p_list, const T& pixel_type);
+  static bool get_debug_info(const char *const s, unsigned int &line_number, unsigned int &file_number);
   static int _levenshtein(const char *const s, const char *const t,
                           gmic_image<int>& d, const int i, const int j);
   static int levenshtein(const char *const s, const char *const t);
-  static bool check_filename(const char *const filename);
   static unsigned int hashcode(const char *const str, const bool is_variable);
   static bool command_has_arguments(const char *const command);
   static const char* basename(const char *const str);
@@ -412,9 +463,9 @@ struct gmic {
   gmic_image<char> status;
 
   float focale3d, light3d_x, light3d_y, light3d_z, specular_lightness3d, specular_shininess3d, _progress, *progress;
-  unsigned long reference_time;
+  gmic_uint64 reference_time;
   unsigned int nb_dowhiles, nb_fordones, nb_repeatdones, nb_carriages, debug_filename, debug_line, cimg_exception_mode;
-  int verbosity,render3d, renderd3d;
+  int verbosity,render3d, renderd3d, network_timeout;
   bool allow_entrypoint, is_change, is_debug, is_running, is_start, is_return, is_quit, is_double3d, is_debug_info,
     _is_abort, *is_abort, is_abort_thread;
   const char *starting_commands_line;
@@ -450,15 +501,31 @@ struct gmic_exception {
 };
 
 template<typename T>
-inline double gmic_mp_call(char *const str, void *const p_list, const T& pixel_type) {
-  return gmic::mp_call(str,p_list,pixel_type);
+inline double gmic_mp_run(char *const str,
+                          void *const p_list, const T& pixel_type) {
+  return gmic::mp_run(str,p_list,pixel_type);
 }
+
 template<typename Ts, typename T>
 inline double gmic_mp_store(const Ts *const ptr,
                             const unsigned int w, const unsigned int h, const unsigned int d, const unsigned int s,
-                            const char *const str, void *const p_list, const T& pixel_type) {
-  return gmic::mp_store(ptr,w,h,d,s,str,p_list,pixel_type);
+                            const bool is_compressed, const char *const str,
+                            void *const p_list, const T& pixel_type) {
+  return gmic::mp_store(ptr,w,h,d,s,is_compressed,str,p_list,pixel_type);
 }
+
+template<typename T, typename Ts>
+inline double gmic_mp_name(const unsigned int ind, Ts *const out_str, const unsigned int siz,
+                           void *const p_list, const T& pixel_type) {
+  return gmic::mp_name(ind,out_str,siz,p_list,pixel_type);
+}
+
+template<typename T>
+inline double gmic_mp_setname(const unsigned int ind, const char *const str,
+                              void *const p_list, const T& pixel_type) {
+  return gmic::mp_setname(ind,str,p_list,pixel_type);
+}
+
 inline bool *gmic_abort_ptr(bool *const p_is_abort) { return gmic::abort_ptr(p_is_abort); }
 
 #endif // #ifndef gmic_version
